@@ -2,9 +2,9 @@ import os
 import shutil
 import sys
 import warnings
+from io import StringIO
 
 import pytest
-from six import StringIO
 
 import setupmeta
 from setupmeta import decode
@@ -21,7 +21,7 @@ os.environ["PYTHONDONTWRITEBYTECODE"] = "1"
 sys.dont_write_bytecode = True
 
 
-def resouce(*relative_path):
+def resource(*relative_path):
     """Full path for 'relative_path'"""
     return os.path.join(TESTS, *relative_path)
 
@@ -68,7 +68,7 @@ def sample_project():
     old_cd = os.getcwd()
     try:
         with setupmeta.temp_resource() as temp:
-            source = resouce("sample")
+            source = resource("sample")
             dest = os.path.join(temp, "sample")
             shutil.copytree(source, dest)
             files = os.listdir(dest)
@@ -278,24 +278,19 @@ def run_internal_setup_py(folder, *args):
 
 
 class MockGit(Git):
-    def __init__(
-        self,
-        dirty=True,
-        describe="v0.1.2-3-g123",
-        branch="master",
-        commitid="abc123",
-        local_tags="",
-        remote_tags="",
-    ):
-        self.dirty = dirty
+    def __init__(self, describe="v0.1.2-3-g123-dirty", branch="master", commitid="abc123", local_tags="", remote_tags=""):
         self.describe = describe
         self.branch = branch
         self.commitid = commitid
-        self.status_message = "## master...origin/master"
+        self.status_message = "## %s...origin/%s" % (branch, branch)
         self._local_tags = local_tags
         self._remote_tags = remote_tags
         Git.__init__(self, TESTS)
 
+    @property
+    def dirty(self):
+        return "-dirty" in self.describe
+
     def get_output(self, cmd, *args, **kwargs):
         if cmd.startswith("diff"):
             return 1 if self.dirty else 0
