@@ -1,8 +1,6 @@
 import os
 import re
-
-from mock import patch
-from six import StringIO
+from unittest.mock import patch
 
 import setupmeta
 from setupmeta.commands import _show_dependencies, DepTree, find_venv
@@ -15,11 +13,9 @@ def run_setup_py(args, expected, folder=None):
     output = conftest.run_setup_py(folder or os.getcwd(), *args)
     for line in expected:
         line = line.strip()
-        if not line:
-            continue
-
-        m = re.search(line, output)
-        assert m, "'%s' not present in output of '%s': %s" % (line, " ".join(args), output)
+        if line:
+            m = re.search(line, output)
+            assert m, "'%s' not present in output of '%s': %s" % (line, " ".join(args), output)
 
 
 def test_check(sample_project):
@@ -45,7 +41,7 @@ def test_check_dependencies():
         run_setup_py(
             ["check", "--deptree"],
             """
-                mock==.+
+                pep440==.+
                 pytest-cov==.+
             """,
             folder=conftest.PROJECT_DIR,
@@ -72,7 +68,6 @@ def test_check_dependencies():
 
 
 class FakeDist:
-
     def __init__(self, spec, requires):
         req = setupmeta.pkg_req(spec)
         self.key = req.key
@@ -82,16 +77,19 @@ class FakeDist:
     def requires(self):
         return self._requires
 
-    @classmethod
-    def from_string(self, specs):
+    @staticmethod
+    def from_string(specs):
         result = []
         for spec in specs.split():
             name, _, req = spec.partition(":")
             if req:
                 req = [setupmeta.pkg_req(r) for r in req.split("+")]
+
             else:
                 req = []
+
             result.append(FakeDist(name, req))
+
         return result
 
 
@@ -229,7 +227,7 @@ def test_version(sample_project):
 
 
 @patch("sys.stdout.isatty", return_value=True)
-@patch("os.popen", return_value=StringIO("60"))
+@patch("os.popen", return_value=conftest.StringIO("60"))
 @patch.dict(os.environ, {"TERM": "testing"})
 def test_console(*_):
     setupmeta.Console._columns = None
@@ -240,21 +238,21 @@ def touch(folder, isdir, *paths):
     for path in paths:
         full_path = os.path.join(folder, path)
         if isdir:
-            os.mkdir(full_path)
+            os.makedirs(full_path)
+
         else:
             with open(full_path, "w") as fh:
                 fh.write("from setuptools import setup\nsetup(setup_requires='setupmeta')\n")
 
 
 def test_clean(sample_project):
-    touch(sample_project, True, ".idea", "build", "dd", "dd/__pycache__", "foo.egg-info")
-    touch(sample_project, False, "foo", "a.pyc", ".pyo", "bar.pyc", "setup.py", "dd/__pycache__/a.pyc")
+    touch(sample_project, True, ".idea", "build", "foo.egg-info", "subfolder/foo/__pycache__")
+    touch(sample_project, False, "subfolder/foo/__pycache__/foo.pyc", "a.pyc", ".pyo", "bar.pyc", "setup.py")
     run_setup_py(
         ["cleanall"],
         """
         deleted build
         deleted foo.egg-info
-        deleted dd.__pycache__
         deleted 2 .pyc files, 1 .pyo files
         """,
     )
